@@ -21,7 +21,7 @@ utilizing normalized mutual information.
 
 Contact: thomas@mandosoft.dev, joe.deweese@lipscomb.edu, kirkdurston@gmail.com
 """
-
+import sys
 import re
 import time
 import csv
@@ -56,9 +56,11 @@ def deweese_schema(df: pd.DataFrame, pattern='^-') -> pd.DataFrame:
     symbols to represent insertions."""
 
     try:
-        df = df.rename(columns=lambda x: x)
         first_row_ix = df.index[0]
-        ix_label = first_row_ix.rsplit('/', 1)
+        if '(' in first_row_ix:
+            ix_label = first_row_ix.rsplit('(', 1)
+        else:
+            ix_label = first_row_ix.rsplit('/', 1)
         ix_label = ix_label[1]
         ix_label = ix_label.rsplit('-', 1)
         df_label = int(ix_label[0])
@@ -78,11 +80,15 @@ def deweese_schema(df: pd.DataFrame, pattern='^-') -> pd.DataFrame:
                     df_label += 1
                 else:
                     column_lab_dict[df.columns[i]] = ''
+
         df = df.rename(columns=column_lab_dict)
-        df = df.drop(columns=[''])
+        if '' in column_lab_dict.values():
+            df = df.drop(columns=[''])
         df = df.rename(columns=lambda x: int(x))
+
     except IndexError or KeyError:
-        exit(1)
+        sys.exit(1)
+
     return df
 
 
@@ -119,6 +125,7 @@ def read_txt_file_format(file) -> pd.DataFrame:
     df = df.replace({'.': '-'})
     df.index.name = 'SEQUENCE_ID'
     df = check_for_duplicates(df)
+    df.fillna('?', inplace=True)
 
     return df
 
@@ -129,6 +136,10 @@ def read_csv_file_format(file) -> pd.DataFrame:
     df = df.rename(columns={df.columns[0]: 'SEQUENCE_ID'})
     df = df.set_index('SEQUENCE_ID', drop=True)
     df = check_for_duplicates(df)
+    df.fillna('?', inplace=True)
+    if df.index[0] == "Domain: Data":
+        df.drop(index=["Domain: Data"], inplace=True)
+    df.columns = range(len(df.columns))
 
     return df
 
@@ -143,10 +154,12 @@ def return_sr_mode(msa: np.ndarray, m_map: dict, c: list, csv_dict: dict, list_s
         i, j = m_map.get(c[0]), m_map.get(c[1])
         max_sum = nmis(msa[:, i], msa[:, j], average_method='geometric')
         new_mode = c
+        """
         if max_sum > 1.0:
             print("\nERROR: Score returned greater than 1.0: ")
             print(c, max_sum)
             exit(1)
+        """
     else:
         A = [[column, []] for column in range(len(c))]
         D = len(A)
@@ -195,15 +208,19 @@ def aggregate(agg: list) -> list:
     while idx < len(agg):
         iex = 0
         while iex < len(agg):
-            # check for intersections
             set1, set2 = set(agg[idx]), set(agg[iex])
+
+            # check for intersections
             if set1 != set2 and set1.intersection(set2):
                 agg[idx] = list(set1.union(set2))
                 del agg[iex]
+                iex -= 1
+
             # check for permutations
             elif set1 == set2:
                 if agg[idx][0] != agg[iex][0]:
                     del agg[iex]
+                    iex -= 1
             else:
                 pass
             iex += 1
@@ -239,11 +256,15 @@ def rectify_sequences(df: pd.DataFrame) -> pd.DataFrame:
     """Corrects symbols in the multiple sequence alignment."""
     alpha = get_unique_elements(df)
     pattern = '[-#?.]'
+    allowed = '[ACDEFGHIKLMNPQRSTVWY]'
+    invalid = '[BJOUXZ]|[bjouxz]'
     for symbol in alpha:
-        if symbol.isupper() and len(symbol) == 1:
+        if re.search(allowed, symbol):
             pass
         else:
-            if symbol.islower() and len(symbol) == 1:
+            if re.search(invalid, symbol):
+                df.replace(symbol, '-', inplace=True)
+            elif symbol.islower() and len(symbol) == 1:
                 df.replace(symbol, symbol.upper(), inplace=True)
             elif re.search(pattern, symbol):
                 df.replace(symbol, '-', inplace=True)
@@ -261,7 +282,7 @@ def encode_msa(df: pd.DataFrame) -> np.ndarray:
     unique integers and returns a numpy matrix"""
     ent = 0
     alpha = get_unique_elements(df)
-    if alpha.size > 22:
+    if alpha.size > 21:
         df = rectify_sequences(df)
         alpha = get_unique_elements(df)
     for symbol in alpha:
@@ -270,7 +291,7 @@ def encode_msa(df: pd.DataFrame) -> np.ndarray:
         else:
             ent += 1
             df.replace(symbol, ent, inplace=True)
-    if ent > 22:
+    if ent > 21:
         print("FAILED: Too many labels encoded: ", ent)
         exit(1)
     print("\nNumber of labels encoded: ", (ent + 1), " including gaps.")
@@ -279,6 +300,32 @@ def encode_msa(df: pd.DataFrame) -> np.ndarray:
     return np_matrix
 
 
+def remove_high_insertion_sites(df: pd.DataFrame, value: int) -> pd.DataFrame:
+    """Useful for removing high insertion columns from MSA data.
+    Returns the pandas dataframe with high insertion attributes removed.
+
+    Provide percentage value to remove as a whole number, i.e. 15 % == 15
+    """
+
+    df.replace({'[-#?.]': None}, regex=True, inplace=True)
+    index_len = len(df.index)
+    null_val = float(value) / 100
+    labels_to_delete = []
+
+    def series_remove_insertions(x):
+        non_nulls = x.count()
+        info_amount = non_nulls / index_len
+        if info_amount < null_val:
+            labels_to_delete.append(x.name)
+
+    # noinspection PyTypeChecker
+    df.apply(series_remove_insertions, axis=0)
+    df.drop(labels_to_delete, axis=1, inplace=True)
+    df.replace({None: '-'}, inplace=True)
+
+    return df
+
+
 def find_clusters(spread: int, df: pd.DataFrame) -> dict:
     """Discovers cluster sites with high shared normalized mutual information.
     Provide a dataframe and a sample spread-width. Returns a dictionary."""
@@ -311,7 +358,7 @@ def find_clusters(spread: int, df: pd.DataFrame) -> dict:
         return_sr_mode(num_msa, msa_map, cluster, csv_dict, hash_list, k)
 
     sorted_list = sorted(hash_list, key=lambda x: x[0], reverse=True)
-    out_list = [x for x in sorted_list if x[0] >= 0.15]
+    out_list = [x for x in sorted_list if x[0] >= 0.10]
     dataframe_label_list = [j for x, j in out_list]
 
     # Check for repeat attributes between pairwise clusters
@@ -322,10 +369,15 @@ def find_clusters(spread: int, df: pd.DataFrame) -> dict:
         return_sr_mode(num_msa, msa_map, cluster, csv_dict, unranked, k)
 
     C = sorted(unranked, key=lambda x: x[0], reverse=True)
-    print("This is C: \n", C)
+
     for x, j in C:
         for col in j:
-            msa_index.remove(col)
+            try:
+                msa_index.remove(col)
+            except ValueError:
+                print("\nFAILED: Tried to remove site location ", col, "but it was not found.\n"
+                       "This is likely due to duplicates not being removed during aggregation.")
+                sys.exit()
 
     print("\nTop ranked clusters:")
     num = 0
